# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.datetime_utils import serialize_datetime
from ..core.pydantic_utilities import deep_union_pydantic_dicts, pydantic_v1


class ContributionResponse(pydantic_v1.BaseModel):
    """
    Represents the cohortized contribution for the Unit Economics for the specified user type.

    This approach assumes all acquisition spend was toward the specified user type.

    This is the core input to deriving certain unit-economic aggregate metrics.
    """

    cohort: str = pydantic_v1.Field()
    """
    The date of the cohort.
    """

    period: int = pydantic_v1.Field()
    """
    The period of the cohort. 0 is the first period.
    """

    cohort_size: int = pydantic_v1.Field()
    """
    The number of users in the cohort.
    """

    cac: typing.Optional[float] = pydantic_v1.Field(default=None)
    """
    The customer acquisition cost (CAC) for the cohort.
    """

    ltv: typing.Optional[float] = pydantic_v1.Field(default=None)
    """
    The revenue lifetime value (LTV) for the cohort.
    """

    ltv_cac: typing.Optional[float] = pydantic_v1.Field(default=None)
    """
    The LTV to CAC ratio for the cohort for the period.
    """

    gmltv: typing.Optional[float] = pydantic_v1.Field(default=None)
    """
    The gross-margin lifetime value, which is the cumululative gross profit, for the cohort for the period.
    """

    gmltv_cac: typing.Optional[float] = pydantic_v1.Field(default=None)
    """
    The gmLTV to CAC ratio for the cohort for the period. This related to the contribution margin.
    """

    contribution: typing.Optional[float] = pydantic_v1.Field(default=None)
    """
    The contribution (gmLTV - CAC) for the cohort for the period.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}
