# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import pydantic_v1
from ..core.request_options import RequestOptions
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.contribution_response import ContributionResponse
from ..types.frequency import Frequency
from ..types.http_validation_error import HttpValidationError
from ..types.user_type import UserType


class UnitEconomicsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_contribution(
        self,
        group_id: int,
        deal_id: int,
        *,
        user_type: UserType,
        freq: typing.Optional[Frequency] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[ContributionResponse]:
        """
        Get the contribution for the user type.

        Parameters
        ----------
        group_id : int

        deal_id : int

        user_type : UserType

        freq : typing.Optional[Frequency]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[ContributionResponse]
            Successful Response

        Examples
        --------
        from termina.client import Termina

        client = Termina(
            token="YOUR_TOKEN",
        )
        client.unit_economics.get_contribution(
            group_id=1,
            deal_id=1,
            user_type="business",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/v1/group/{jsonable_encoder(group_id)}/deal/{jsonable_encoder(deal_id)}/data/contribution",
            method="GET",
            params={"user_type": user_type, "freq": freq},
            request_options=request_options,
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(typing.List[ContributionResponse], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncUnitEconomicsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_contribution(
        self,
        group_id: int,
        deal_id: int,
        *,
        user_type: UserType,
        freq: typing.Optional[Frequency] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[ContributionResponse]:
        """
        Get the contribution for the user type.

        Parameters
        ----------
        group_id : int

        deal_id : int

        user_type : UserType

        freq : typing.Optional[Frequency]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[ContributionResponse]
            Successful Response

        Examples
        --------
        from termina.client import AsyncTermina

        client = AsyncTermina(
            token="YOUR_TOKEN",
        )
        await client.unit_economics.get_contribution(
            group_id=1,
            deal_id=1,
            user_type="business",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/v1/group/{jsonable_encoder(group_id)}/deal/{jsonable_encoder(deal_id)}/data/contribution",
            method="GET",
            params={"user_type": user_type, "freq": freq},
            request_options=request_options,
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(typing.List[ContributionResponse], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)
